@@ -12,13 +12,13 @@ use super::*;
 /// at the underlying `BufferedReader`.
 #[derive(Debug)]
 pub struct Dup<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> {
-    reader: T,
-
     // The number of bytes that have been consumed.
     cursor: usize,
 
     // The user settable cookie.
     cookie: C,
+
+    reader: T,
 }
 
 assert_send_and_sync!(Dup<T, C>
@@ -73,7 +73,7 @@ impl<T: BufferedReader<C>, C: fmt::Debug + Sync + Send> io::Read for Dup<T, C> {
         let data = &data[self.cursor..];
 
         let amount = cmp::min(buf.len(), data.len());
-        buf.copy_from_slice(&data[..amount]);
+        buf[..amount].copy_from_slice(&data[..amount]);
 
         self.cursor += amount;
 
