@@ -5,17 +5,15 @@ use std::fmt;
 use crate::BufferedReader;
 
 /// Always returns EOF.
+#[derive(Debug)]
 pub struct EOF<C> {
     cookie: C,
 }
 
-impl<C> fmt::Display for EOF<C> {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "EOF")
-    }
-}
+assert_send_and_sync!(EOF<C>
+                      where C: fmt::Debug);
 
-impl<C> fmt::Debug for EOF<C> {
+impl<C> fmt::Display for EOF<C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("EOF")
             .finish()
@@ -35,7 +33,7 @@ impl<C> EOF<C> {
     /// Instantiates a new `EOF` with a cookie.
     pub fn with_cookie(cookie: C) -> Self {
         EOF {
-            cookie: cookie,
+            cookie,
         }
     }
 }
@@ -46,7 +44,7 @@ impl<C> Read for EOF<C> {
     }
 }
 
-impl<C> BufferedReader<C> for EOF<C> {
+impl<C: fmt::Debug + Sync + Send> BufferedReader<C> for EOF<C> {
     fn buffer(&self) -> &[u8] {
         return &b""[..];
     }
