@@ -4,16 +4,17 @@
 //! performance of the statistics example by ~10% over the
 //! Generic.
 
-use libc::{c_void, size_t, mmap, munmap, PROT_READ, MAP_PRIVATE};
+use libc::{mmap, munmap, PROT_READ, MAP_PRIVATE};
 use std::fmt;
 use std::fs;
 use std::io;
 use std::os::unix::io::AsRawFd;
 use std::slice;
-use std::path::Path;
+use std::path::{Path, PathBuf};
 use std::ptr;
 
 use super::*;
+use crate::file_error::FileError;
 
 // For small files, the overhead of manipulating the page table is not
 // worth the gain.  This threshold has been chosen so that on my
@@ -24,45 +25,49 @@ const MMAP_THRESHOLD: u64 = 16 * 4096;
 ///
 /// This implementation tries to mmap the file, falling back to
 /// just using a generic reader.
-pub struct File<'a, C>(Imp<'a, C>);
+pub struct File<'a, C: fmt::Debug + Sync + Send>(Imp<'a, C>, PathBuf);
 
-impl<'a, C> fmt::Display for File<'a, C> {
+assert_send_and_sync!(File<'_, C>
+                      where C: fmt::Debug);
+
+impl<'a, C: fmt::Debug + Sync + Send> fmt::Display for File<'a, C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", self.0)
+        write!(f, "{} {:?}", self.0, self.1.display())
     }
 }
 
-impl<'a, C> fmt::Debug for File<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> fmt::Debug for File<'a, C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_tuple("File")
             .field(&self.0)
+            .field(&self.1)
             .finish()
     }
 }
 
 /// The implementation.
-enum Imp<'a, C> {
+enum Imp<'a, C: fmt::Debug + Sync + Send> {
     Generic(Generic<fs::File, C>),
     MMAP {
-        addr: *mut c_void,
-        length: size_t,
         reader: Memory<'a, C>,
     }
 }
 
-impl<'a, C> Drop for Imp<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> Drop for Imp<'a, C> {
     fn drop(&mut self) {
         match self {
             Imp::Generic(_) => (),
-            Imp::MMAP { addr, length, .. } =>
+            Imp::MMAP { reader, } => {
+                let buf = reader.source_buffer();
                 unsafe {
-                    munmap(*addr, *length);
-                },
+                    munmap(buf.as_ptr() as *mut _, buf.len());
+                }
+            },
         }
     }
 }
 
-impl<'a, C> fmt::Display for Imp<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> fmt::Display for Imp<'a, C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "File(")?;
         match self {
@@ -73,17 +78,17 @@ impl<'a, C> fmt::Display for Imp<'a, C> {
     }
 }
 
-impl<'a, C> fmt::Debug for Imp<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> fmt::Debug for Imp<'a, C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
             Imp::Generic(ref g) =>
                 f.debug_tuple("Generic")
                 .field(&g)
                 .finish(),
-            Imp::MMAP { ref addr, ref length, ref reader } =>
+            Imp::MMAP { reader, } =>
                 f.debug_struct("MMAP")
-                .field("addr", addr)
-                .field("length", length)
+                .field("addr", &reader.source_buffer().as_ptr())
+                .field("length", &reader.source_buffer().len())
                 .field("reader", reader)
                 .finish(),
         }
@@ -97,17 +102,20 @@ impl<'a> File<'a, ()> {
     }
 }
 
-impl<'a, C> File<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> File<'a, C> {
     /// Like `open()`, but sets a cookie.
     pub fn with_cookie<P: AsRef<Path>>(path: P, cookie: C) -> io::Result<Self> {
+        let path = path.as_ref();
+
         // As fallback, we use a generic reader.
         let generic = |file, cookie| {
             Ok(File(
                 Imp::Generic(
-                    Generic::with_cookie(file, None, cookie))))
+                    Generic::with_cookie(file, None, cookie)),
+                path.into()))
         };
 
-        let file = fs::File::open(path)?;
+        let file = fs::File::open(path).map_err(|e| FileError::new(path, e))?;
 
         // For testing and benchmarking purposes, we use the variable
         // SEQUOIA_DONT_MMAP to turn off mmapping.
@@ -115,7 +123,8 @@ impl<'a, C> File<'a, C> {
             return generic(file, cookie);
         }
 
-        let length = file.metadata()?.len();
+        let length =
+            file.metadata().map_err(|e| FileError::new(path, e))?.len();
 
         // For small files, the overhead of manipulating the page
         // table is not worth the gain.
@@ -144,24 +153,23 @@ impl<'a, C> File<'a, C> {
 
         Ok(File(
             Imp::MMAP {
-                addr,
-                length,
                 reader: Memory::with_cookie(slice, cookie),
-            }
+            },
+            path.into(),
         ))
     }
 }
 
-impl<'a, C> io::Read for File<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> io::Read for File<'a, C> {
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
         match self.0 {
             Imp::Generic(ref mut reader) => reader.read(buf),
             Imp::MMAP { ref mut reader, .. } => reader.read(buf),
-        }
+        }.map_err(|e| FileError::new(&self.1, e))
     }
 }
 
-impl<'a, C> BufferedReader<C> for File<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> BufferedReader<C> for File<'a, C> {
     fn buffer(&self) -> &[u8] {
         match self.0 {
             Imp::Generic(ref reader) => reader.buffer(),
@@ -170,17 +178,19 @@ impl<'a, C> BufferedReader<C> for File<'a, C> {
     }
 
     fn data(&mut self, amount: usize) -> io::Result<&[u8]> {
+        let path = &self.1;
         match self.0 {
             Imp::Generic(ref mut reader) => reader.data(amount),
             Imp::MMAP { ref mut reader, .. } => reader.data(amount),
-        }
+        }.map_err(|e| FileError::new(path, e))
     }
 
     fn data_hard(&mut self, amount: usize) -> io::Result<&[u8]> {
+        let path = &self.1;
         match self.0 {
             Imp::Generic(ref mut reader) => reader.data_hard(amount),
             Imp::MMAP { ref mut reader, .. } => reader.data_hard(amount),
-        }
+        }.map_err(|e| FileError::new(path, e))
     }
 
     fn consume(&mut self, amount: usize) -> &[u8] {
@@ -191,17 +201,19 @@ impl<'a, C> BufferedReader<C> for File<'a, C> {
     }
 
     fn data_consume(&mut self, amount: usize) -> io::Result<&[u8]> {
+        let path = &self.1;
         match self.0 {
             Imp::Generic(ref mut reader) => reader.data_consume(amount),
             Imp::MMAP { ref mut reader, .. } => reader.data_consume(amount),
-        }
+        }.map_err(|e| FileError::new(path, e))
     }
 
     fn data_consume_hard(&mut self, amount: usize) -> io::Result<&[u8]> {
+        let path = &self.1;
         match self.0 {
             Imp::Generic(ref mut reader) => reader.data_consume_hard(amount),
             Imp::MMAP { ref mut reader, .. } => reader.data_consume_hard(amount),
-        }
+        }.map_err(|e| FileError::new(path, e))
     }
 
     fn get_mut(&mut self) -> Option<&mut dyn BufferedReader<C>> {
@@ -238,3 +250,15 @@ impl<'a, C> BufferedReader<C> for File<'a, C> {
         }
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    #[test]
+    fn error_contains_path() {
+        let p = "/i/do/not/exist";
+        let e = File::open(p).unwrap_err();
+        assert!(e.to_string().contains(p));
+    }
+}
