@@ -223,6 +223,8 @@
 //! [`BufReader`]: https://doc.rust-lang.org/stable/std/io/struct.BufReader.html
 //! [HTTP's chunk transfer encoding]: https://en.wikipedia.org/wiki/Chunked_transfer_encoding
 
+#![doc(html_favicon_url = "https://docs.sequoia-pgp.org/favicon.png")]
+#![doc(html_logo_url = "https://docs.sequoia-pgp.org/logo.svg")]
 #![warn(missing_docs)]
 
 use std::io;
