@@ -11,7 +11,8 @@ use super::*;
 /// Although it is possible to use `Generic` to wrap a
 /// buffer, this implementation is optimized for a memory buffer, and
 /// avoids double buffering.
-pub struct Memory<'a, C> {
+#[derive(Debug)]
+pub struct Memory<'a, C: fmt::Debug + Sync + Send> {
     buffer: &'a [u8],
     // The next byte to read in the buffer.
     cursor: usize,
@@ -20,22 +21,16 @@ pub struct Memory<'a, C> {
     cookie: C,
 }
 
-impl<'a, C> fmt::Display for Memory<'a, C> {
+assert_send_and_sync!(Memory<'_, C>
+                      where C: fmt::Debug);
+
+impl<'a, C: fmt::Debug + Sync + Send> fmt::Display for Memory<'a, C> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "Memory ({} of {} bytes read)",
                self.cursor, self.buffer.len())
     }
 }
 
-impl<'a, C> fmt::Debug for Memory<'a, C> {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        f.debug_struct("Memory")
-            .field("buffer (bytes)", &&self.buffer.len())
-            .field("cursor", &self.cursor)
-            .finish()
-    }
-}
-
 impl<'a> Memory<'a, ()> {
     /// Instantiates a new `Memory`.
     ///
@@ -45,16 +40,16 @@ impl<'a> Memory<'a, ()> {
     }
 }
 
-impl<'a, C> Memory<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> Memory<'a, C> {
     /// Like `new()`, but sets a cookie.
     ///
     /// The cookie can be retrieved using the `cookie_ref` and
     /// `cookie_mut` methods, and set using the `cookie_set` method.
     pub fn with_cookie(buffer: &'a [u8], cookie: C) -> Self {
         Memory {
-            buffer: buffer,
+            buffer,
             cursor: 0,
-            cookie: cookie,
+            cookie,
         }
     }
 
@@ -63,9 +58,14 @@ impl<'a, C> Memory<'a, C> {
     pub fn total_out(&self) -> usize {
         return self.cursor;
     }
+
+    #[allow(dead_code)]
+    pub(crate) fn source_buffer(&self) -> &[u8] {
+        self.buffer
+    }
 }
 
-impl<'a, C> io::Read for Memory<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> io::Read for Memory<'a, C> {
     fn read(&mut self, buf: &mut [u8]) -> Result<usize, io::Error> {
         let amount = cmp::min(buf.len(), self.buffer.len() - self.cursor);
         buf[0..amount].copy_from_slice(
@@ -75,7 +75,7 @@ impl<'a, C> io::Read for Memory<'a, C> {
     }
 }
 
-impl<'a, C> BufferedReader<C> for Memory<'a, C> {
+impl<'a, C: fmt::Debug + Sync + Send> BufferedReader<C> for Memory<'a, C> {
     fn buffer(&self) -> &[u8] {
         &self.buffer[self.cursor..]
     }
